#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define DEFAULT_FONT	"lucidasanstypewriter-bold-18"
#define AUTO_REPEAT	8
#define REPEAT_SPEED	1
#define TOTAL_COLORS	10

extern int RGB[TOTAL_COLORS][3];

extern int X,Y,NEXT,PIECE;
extern unsigned char LEFT,RIGHT,DOWN,MOUSE_X,MOUSE_Y;
int KBD_COUNT=0;

int WIDTH=550,HEIGHT=500,FONT_HEIGHT;

int GAME=0,PAUSE=0,PANIC=0,milliseconds;

void CheckArgs();

main(int argc,char **argv)
{
	Display *disp;
	Window win,root;
	int screen,black,EXIT;
	GC gc;
	XEvent event;
	XSizeHints wmhints;
	XFontStruct *font;
	int hz;

	CheckArgs(argc,argv);

	read_tetrisrc();

	NEXT=rand()%7+1;

	hz=FindHz();
	milliseconds=10000*hz/50;

	if ((disp=XOpenDisplay(NULL)) == NULL)
	{
		perror("XOpenDisplay");
		exit(0);
	}

	if ((font=XLoadQueryFont(disp,DEFAULT_FONT)) == NULL)
	{
		perror("XLoadQueryFont");
		exit(0);
	}
	FONT_HEIGHT=font->max_bounds.ascent+font->max_bounds.descent;

	root=DefaultRootWindow(disp);
	screen=DefaultScreen(disp);
	black=BlackPixel(disp,screen);

	clear_board();

	win=XCreateSimpleWindow(disp,root,0,0,WIDTH,HEIGHT,1,black,black);

	wmhints.flags=USPosition|USSize|PMinSize|PMaxSize;
	wmhints.x=wmhints.y=0;
	wmhints.flags=PSize|PMinSize|PMaxSize;
	wmhints.width=wmhints.min_width=wmhints.max_width=WIDTH;
	wmhints.height=wmhints.min_height=wmhints.max_height=HEIGHT;
	XSetWMNormalHints(disp,win,&wmhints);
	
	XStoreName(disp,win,"Yet another xtetris   by \"af\"");
	XMapWindow(disp,win);

	gc=XCreateGC(disp,win,0,NULL);

	XSelectInput(disp,win,	ExposureMask|
				ResizeRedirectMask|
				KeyPressMask|
				KeyReleaseMask|
				ButtonPressMask|
				Button1MotionMask);

	XSync(disp,False);

	for (;;)
	{
		GetKeyboardState(disp);
		if (GAME && !PAUSE)
		{
			if (PANIC) panic_stuff(disp,win,gc);
			if (LEFT)
			{
				if (KBD_COUNT == 0) move_left(disp,win,gc);
				KBD_COUNT++;
				if (KBD_COUNT >= AUTO_REPEAT)
				if (!((KBD_COUNT-AUTO_REPEAT)%REPEAT_SPEED))
				move_left(disp,win,gc);
			}
                        if (RIGHT)
                        {
                                if (KBD_COUNT == 0) move_right(disp,win,gc);
                                KBD_COUNT++;
                                if (KBD_COUNT >= AUTO_REPEAT)
                                if (!((KBD_COUNT-AUTO_REPEAT)%REPEAT_SPEED))
                                move_right(disp,win,gc);
                        }                                                                          
                        if (DOWN)
                        {
                                if (KBD_COUNT == 0) move_down(disp,win,gc,font);
                                KBD_COUNT++;
                                if (KBD_COUNT >= AUTO_REPEAT)
                                if (!((KBD_COUNT-AUTO_REPEAT)%REPEAT_SPEED))
                                move_down(disp,win,gc,font);
                        }                                                                          
			do_gravity(disp,win,gc,font);
		}
		usleep(milliseconds);

		if (XPending(disp) == 0) continue;

		XNextEvent(disp,&event);

		switch (event.type)
		{
		case Expose:
			if (event.xexpose.count) break;
			Redraw(disp,win,gc);
			if (!GAME)
				show_message(disp,win,gc,font,"Press 'S' to start game",1);
			if (PAUSE)
				show_message(disp,win,gc,font,"Paused. Press 'P' to resume",1);
			break;

		case ResizeRequest:
			WIDTH=event.xresizerequest.width;
			HEIGHT=event.xresizerequest.height;
			Redraw(disp,win,gc);
			break;

		case KeyPress:
		case KeyRelease:
			ProcessKeyEvent(disp,win,gc,&event,font);
			break;

		case MotionNotify:
		case ButtonPress:
			ProcessMouseEvent(disp,win,gc,&event);
			break;
		}
	}

	XDestroyWindow(disp,win);
	XFree(gc);
}

void CheckArgs(int argc,char **argv)
{
	int i,j;
	for (i=1; i<argc; i++)
	{
		if (strcmp(argv[i],"-customize") == 0) customize_keyboard(); else
		
		if (strcmp(argv[i],"-mono") == 0)
		{
			float sum;
			for (j=0; j<TOTAL_COLORS; j++)
			{
				sum=(float)RGB[j][0]*0.3+(float)RGB[j][1]*0.59+(float)RGB[j][2]*0.11;
				RGB[j][0]=RGB[j][1]=RGB[j][2]=(65535-(int)sum)/2+(int)sum;
				RGB[0][0]=RGB[0][1]=RGB[0][2]=RGB[9][0]=RGB[9][1]=RGB[9][2]=0;
			}
		}
		  else
		fprintf(stderr,"Unrecognized option: %s\n",argv[i]);
	}
}
